# Copyright (C) 2016  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from mir.winenv import configlib

HELP = 'Add an environment.'


def setup_parser(subparsers):
    parser = subparsers.add_parser(
        'add',
        description=HELP,
        help=HELP,
    )
    parser.add_argument('--config',
                        default=configlib.CONFIG_PATH,
                        help='Configuration file to use.')
    parser.add_argument('name', help='Name of environment')
    parser.add_argument('-p', '--prefix', help='Wine prefix path')
    parser.add_argument('-a', '--arch', help='Windows architecture')
    parser.add_argument('-l', '--lang', help='Environment LANG')
    parser.set_defaults(func=main)


def main(args):
    config = configlib.load_config(args.config)
    name = args.name
    if name not in config:
        config.add_section(name)
    if args.prefix is None:
        args.prefix = f'~/.local/share/wineprefixes/{args.name}'
    config[name]['prefix'] = args.prefix
    if args.arch is not None:
        config[name]['arch'] = args.arch
    if args.lang is not None:
        config[name]['lang'] = args.lang
    configlib.save_config(config, args.config)



# Copyright (C) 2016, 2017  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import importlib


def setup_parser(parser):
    """Setup ArgumentParser with commands."""
    subparsers = parser.add_subparsers(title='Commands')
    for command in COMMANDS:
        command.setup_parser(subparsers)


COMMANDS = []


def _add_command_module(module_name: str):
    """Add a submodule as a command."""
    mod = importlib.import_module(f'mir.winenv.commands.{module_name}')
    COMMANDS.append(mod)


_add_command_module('add')
_add_command_module('list')
_add_command_module('run')



# Copyright (C) 2016  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from mir.winenv import configlib

HELP = 'List environments.'


def setup_parser(subparsers):
    parser = subparsers.add_parser(
        'list',
        description=HELP,
        help=HELP,
    )
    parser.add_argument('--config',
                        default=configlib.CONFIG_PATH,
                        help='Configuration file to use.')
    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        help='List environment settings.')
    parser.set_defaults(func=main)

_KEYS = ['prefix', 'arch', 'lang']


def main(args):
    config = configlib.load_config(args.config)
    if args.verbose:
        sections = []
        for section in config.sections():
            lines = []
            lines.append(section)
            for key in _KEYS:
                lines.append('{}={}'.format(key, config[section][key]))
            sections.append('\n'.join(lines))
        print('\n\n'.join(sections))
    else:
        for section in config.sections():
            print(section)



# Copyright (C) 2016, 2017  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import configparser
import os
from pathlib import Path


CONFIG_PATH = (Path.home() / '.config' / 'winenv' / 'config.ini')


def make_config():
    """Make default configuration."""
    config = configparser.ConfigParser()
    config['DEFAULT']['arch'] = 'win32'
    config['DEFAULT']['lang'] = 'en_US.UTF-8'
    return config


def load_config(path: 'PathLike'):
    """Safely read config file."""
    path = Path(path)
    config = make_config()
    if path.is_file():
        config.read(path)
    return config


def save_config(config, path: 'PathLike'):
    """Safely save config file."""
    path = Path(path)
    path.parent.mkdir(parents=True, exist_ok=True)
    with path.open('w') as file:
        config.write(file)


def get_env_vars(config, env_name):
    """Return a dict of environment variables for the named environment."""
    if not config.has_section(env_name):
        raise ValueError(f"{env_name} environment doesn't exist")
    config_section = config[env_name]
    return {
        'WINEPREFIX': os.path.expanduser(config_section['prefix']),
        'WINEARCH': config_section['arch'],
        'LANG': config_section['lang'],
    }



# Copyright (C) 2016, 2017  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

import argparse
import logging
import sys

from mir.winenv import commands


def main(args):
    """Entry point."""
    try:
        logging.basicConfig(level='INFO')
        parser = argparse.ArgumentParser()
        commands.setup_parser(parser)
        args = parser.parse_args(args)
        func = args.func
        func(args)
    except SystemExit as e:
        return e.code
    else:
        return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# Copyright (C) 2016, 2017  Allen Li

# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from setuptools import setup

setup(
    name='mir.winenv',
    version='0.4.0',
    description='Manage Wine environments',
    long_description='',
    keywords='',
    url='https://github.com/darkfeline/mir.winenv',
    author='Allen Li',
    author_email='darkfeline@felesatra.moe',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
    ],

    packages=['mir.winenv'],
)
